#include <math.h>
#include <time.h>
#include "matrix.h"
#include "vector.h"
#include "bscurv.h"
#include "bssurf.h"

#define MIN(x,y)	((x) < (y)) ? (x) : (y)
#define PX (*m_pXcoord)
#define PY (*m_pYcoord)
#define PZ (*m_pZcoord)

BsSurf::BsSurf(int n, int p, Vector& U, int m, int q, Vector& V,
	     Matrix& x, Matrix& y, Matrix& z)
{

	_n = n; _p = p; _m = m; _q = q;

	m_pXcoord = new Matrix(x.GetRowSize() , x.GetColSize());
	(*m_pXcoord) = x;

	m_pYcoord = new Matrix(y.GetRowSize() , y.GetColSize());
	(*m_pYcoord) = y;

	m_pZcoord = new Matrix(z.GetRowSize() , z.GetColSize());
	(*m_pZcoord) = z;

	Matrix empty(3,3);

	m_pUdir = new BsCurv(n, p, empty, U);
	m_pVdir = new BsCurv(m, q, empty, V);
}

BsSurf::~BsSurf(void)
{
	delete m_pXcoord, m_pYcoord, m_pZcoord;
	delete m_pUdir, m_pVdir;
}

void BsSurf::SurfacePoint(double u, double v, Vector &S)
{
	int uspan, vspan, uind, vind, l, k, n, p, m, q;

	double tempx, tempy, tempz;

	n = _n; p = _p; m = _m; q = _q;

	Vector Nu(p+1), Nv(q+1);


	uspan = m_pUdir->FindSpan(u);
	m_pUdir->BasisFuns(uspan, u, Nu);
	vspan = m_pVdir->FindSpan(v);
	m_pVdir->BasisFuns(vspan, v, Nv);


	std::cout << Nv << std::endl;

	uind = uspan - p;
	S[0] = S[1] = S[2] = 0.0;

	for(l = 0; l <= q; l++)
	{
		tempx = 0.0; tempy = 0.0; tempz = 0.0;
		vind = vspan-q+l;
		for(k = 0; k <= p; k++)
		{
			tempx = tempx + Nu[k]*PX[uind+k][vind];
			tempy = tempy + Nu[k]*PY[uind+k][vind];
			tempz = tempz + Nu[k]*PZ[uind+k][vind];
		}
		S[0] = S[0] + Nv[l]*tempx;
		S[1] = S[1] + Nv[l]*tempy;
		S[2] = S[2] + Nv[l]*tempz;
	}
}

void BsSurf::SurfaceDerivsAlg1(double u, double v, int d, Matrix** SKL)
{
	int p, n, q, m;
	int k,l,r,s;
	int uspan, vspan, dd;
	p = _p; n = _n; q = _q; m = _m;
	Vector tempx(q+1), tempy(q+1), tempz(q+1);

	int du = MIN(d,p);
	for( k = p + 1; k <= d; k++)
		for( l = 0; l <= d-k; l++)
		{
			(*(SKL[0]))[k][l] = 0.0;
			(*(SKL[1]))[k][l] = 0.0;
			(*(SKL[2]))[k][l] = 0.0;
		}

	int dv = MIN(d,q);
	for( l = q + 1; l <= d; l++)
		for( k = 0; k <= d-l; k++)	
		{
			(*(SKL[0]))[k][l] = 0.0;
			(*(SKL[1]))[k][l] = 0.0;
			(*(SKL[2]))[k][l] = 0.0;
		}


	Matrix Nu(du+1, p+1), Nv(dv+1, q+1);


	uspan = m_pUdir->FindSpan(u);
	m_pUdir->DerBasisFuns(uspan, u, du, Nu);
	vspan = m_pVdir->FindSpan(v);
	m_pVdir->DerBasisFuns(vspan, v, dv, Nv);

	//cout << "Nu" << Nu << endl;
	//cout << "Nv" << Nv << endl;

	for(k = 0; k <= du; k++)
	{
		for(s = 0; s <= q; s++)
		{
			tempx[s] = tempy[s] = tempz[s] = 0.0;
			for(r = 0; r <= p; r++)
			{
				tempx[s] = tempx[s] + Nu[k][r]*PX[uspan-p+r][vspan-q+s];
				tempy[s] = tempy[s] + Nu[k][r]*PY[uspan-p+r][vspan-q+s];
				tempz[s] = tempz[s] + Nu[k][r]*PZ[uspan-p+r][vspan-q+s];
			}
		}

		dd = MIN(d-k, dv);
		for(l = 0; l <= dd; l++)
		{
			(*(SKL[0]))[k][l] = (*(SKL[1]))[k][l] = (*(SKL[2]))[k][l] = 0.0;
			for(s = 0; s <= q; s++)
			{
				(*(SKL[0]))[k][l] = (*(SKL[0]))[k][l] + Nv[l][s]*tempx[s];
				(*(SKL[1]))[k][l] = (*(SKL[1]))[k][l] + Nv[l][s]*tempy[s];
				(*(SKL[2]))[k][l] = (*(SKL[2]))[k][l] + Nv[l][s]*tempz[s];
			}
		}
	}
}

void BsSurf::GetUdirDerivsCP(int i, Matrix& CP)
{
	int j,m,p;
	Vector& U = *(m_pUdir->_pKnot);

	p = _p;
	m = PX.GetColSize();
	for(j = 0 ; j < m; j++){
		CP[j][0] = p * ( (PX[i+1][j] - PX[i][j]) / (U[i+p+1] - U[i+1]) );
		CP[j][1] = p * ( (PY[i+1][j] - PY[i][j]) / (U[i+p+1] - U[i+1]) );
		CP[j][2] = p * ( (PZ[i+1][j] - PZ[i][j]) / (U[i+p+1] - U[i+1]) );
	}
}

void RearrangeforOnecoord(Matrix** from, Matrix** to, int size)
{
	//두개의 포인터가 유효한가를 검사하는 코드가 필요
	//일단 생략
	int i,j;

	for(j = 0; j < size; j++)
		for(i = 0; i < 3; i++)
			to[i]->InsertRow(from[j]->ExtractCol(i), j);
}

