#include <iostream>
#include <fstream>
#include <time.h>
#include "matrix.h"
#include "vector.h"
#include "bscurv.h"
#include "bssurf.h"

std::ofstream main_result("main_result.txt", std::ios_base::out);
std::ofstream Q_data("Q_data.txt", std::ios_base::out);
std::ofstream surfacev("surfacec_v.txt", std::ios_base::out);
std::ofstream surfaceu("surfacecp_u.txt", std::ios_base::out);
std::ofstream processinfo("processinfo.txt", std::ios_base::out);
std::ofstream surface("surface.txt", std::ios_base::out);
std::fstream fin("./21curve_torso.txt", std::ios_base::in);

void LeastSquareProcess(BsCurv* LSC, Matrix** Q, int p, double tol, int num_curve);
void KnotMergeProcess(BsCurv* LSC, int p, int num_curve);
void UdirInterpolProcess(BsCurv* LSC, Matrix** UCP, BsCurv& Vinter, int num_curve);
void SaveSRF(BsSurf& SRF);

int main(void)
{
	int num_curve, p, c, icp;
	int i, j, r;
	double x,y,z;
	double tol = 0.001;
	double tol2 = 0.001;
	Vector P(3);
	Vector C(3);

	Matrix::SetSciFi();
	Vector::SetSciFi();

	fin >> num_curve;
		
	p = 3;
	c = 3;

	Matrix** Q;
	Q = new Matrix* [num_curve];
	
	for(i = 0; i < num_curve; i++)
	{
		fin >> r;	
		Q[i] = new Matrix(r,c);
		
		for(j = 0; j < r; j++)
		{
			fin >> x >> y >> z;
			(*(Q[i]))[j][0] = x;
			(*(Q[i]))[j][1] = y;
			(*(Q[i]))[j][2] = z;
		}
	}
	
	BsCurv* LSC = new BsCurv[num_curve];
	
	//단면곡선 생성
	LeastSquareProcess(LSC,Q,p,tol,num_curve);
	
	if(num_curve < 4) //곡선이 4개는 되야 skinning한다
	{
		std::cout << "Program is terminated!!" << std::endl;
		return 0;
	}
	
	//노트 병합
	KnotMergeProcess(LSC, p, num_curve);
	
	//늘어난 노트 제거
	RemoveKnotSectionCurves(LSC, num_curve, Q, tol2);

	//마지막으로 노트제거된 곡선을 파일에 기록한다
	for(i = 0; i < num_curve; i++)
	{
		main_result << "Pwre=[" << std::endl;
		main_result << *(LSC[i]._pCtrPoint);
		main_result << "];" << std::endl;
		main_result << "Ure=[" << std::endl;
		main_result << *(LSC[i]._pKnot) << std::endl;
		main_result << "];" << std::endl;
	}
	
	
	//v방향 보간
	icp = LSC[0]._n+1;
	Matrix** UCP = new Matrix* [num_curve];
	for(i = 0; i < num_curve; i++)
		UCP[i] = new Matrix(icp, 3);
	BsCurv Uinter(num_curve-1/*_n*/, 3/*degree*/);

	UdirInterpolProcess(LSC, UCP, Uinter, num_curve);


	//make a surface object
	Matrix** SCD = new Matrix* [3];
	for(i = 0; i < 3; i++)
		SCD[i] = new Matrix(num_curve,icp);


	RearrangeforOnecoord(UCP, SCD, num_curve);

	processinfo << "UCP" << std::endl;
	for(i = 0; i < num_curve; i++)
		processinfo << *(UCP[i]) << std::endl;

	processinfo << "SCD" << std::endl;
	for(i = 0; i < 3; i++)
		processinfo << *(SCD[i]) << std::endl;

	BsSurf SRF(Uinter._n, Uinter._p, (*Uinter._pKnot), 
	           LSC[0]._n, LSC[0]._p, *(LSC[0]._pKnot), 
	          (*SCD[0]), (*SCD[1]), (*SCD[2])
		   );


	SaveSRF(SRF);
	
	//SRFu(1,v)의 조정점을 구한다.
	//Matrix CP( SRF._m ,3);
	//SRF.GetUdirDerivsCP(1,CP);

	//surface << "Fu(1,v) 의 조정점" << endl;
	//surface << CP << endl;
	//surface << SRF.m_pVdir->_pKnot << endl;

	//////////////////////////////////////////////////////////////////////////
	//memory release
	delete [] LSC;

	for(i = 0; i < num_curve; i++)
			delete Q[i];
	delete [] Q;
	
	for(i = 0; i < num_curve; i++)
		delete UCP[i];
	delete [] UCP;

	for(i = 0; i < 3; i++)
	{
		delete SCD[i];
//		delete SKL[i];
	}
	delete [] SCD;
//	delete [] SKL;

	std::cout << "Program is terminated!!" << std::endl;

	main_result.close();
	return 0;
}



void LeastSquareProcess(BsCurv* LSC, Matrix** Q, int p, double tol, int num_curve)
{
	//clock_t start, finish;
	//double duration;

	int i;
	for(i = 0; i < num_curve; i++)
		{
		//start = clock();
		LSC[i].LeastSquareCurveErrBnd(*(Q[i]), p, tol);
		//processinfo << *(LSC[i]._pError) << endl;
		
		//cout << "Error estimation of this curve" << endl;
		//LSC[i].RemoveCurveKnot4App(*(Q[i]), tol);
		//processinfo << *(LSC[i]._pError) << endl;

		//finish = clock();
		//duration = (double)(finish - start) / CLOCKS_PER_SEC;
		
		std::cout << "It have spent " << "Linux" <<
			" seconds to approximate " << i << 
			"curve within given tolerance" << std::endl;
		
		std::cout << "make G1-continuity" << std::endl;
		LSC[i].Colinear();
		std::cout << "Final control point : " << LSC[i]._n + 1 << std::endl;
		
		
		std::cout << i << " curve was completed" << std::endl;
		std::cout << "************************************************" << std::endl;
	//	processinfo << *(LSC[i]._pError) << endl;
	
	}
	std::cout << "Terminated Least Square Process" << std::endl;
}

void KnotMergeProcess(BsCurv* LSC, int p, int num_curve)
{
	MergeKnotVect(LSC/*nurbs list*/, num_curve/*length of list*/, 3/*p*/);

	std::cout << "Terminated Knot Merge Process" << std::endl;
	std::cout << "Merged Control Point : " << LSC[0]._n+1 << std::endl;
}


void UdirInterpolProcess(BsCurv* LSC, Matrix** UCP, BsCurv& Uinter, int num_curve)
{
	int i, j, k;

	Vector Udir(num_curve/*size of LSC*/);
	UdirKnotParam(LSC, num_curve, Udir);

	
	//각 조정점을 v방향으로 묶어 데이터셋을 만들어낸다.
	Matrix PQ(num_curve,3);
	
	
	int icp = LSC[0]._n+1;

	
	for(i = 0; i < icp; i++) //조정점 번호
	{
		for(j = 0; j < num_curve; j++) //곡선번호
		{
			PQ[j][0] = (*(LSC[j]._pCtrPoint))[i][0];
			PQ[j][1] = (*(LSC[j]._pCtrPoint))[i][1];
			PQ[j][2] = (*(LSC[j]._pCtrPoint))[i][2];
		}//조정점을 점데이터꼴로 만들었다.
		
		Uinter.InterpolCurve(PQ, &Udir); //보간

		//v방향 control point  num_curve개 만큼 출력한다
		surfacev << i << " : " << '\n' << *(Uinter._pCtrPoint) << std::endl;

		//u방향 중심으로 조정점들을 제 배열한다
		for(k = 0 ; k < num_curve; k++)
		{
			(*(UCP[k]))[i][0] = (*(Uinter._pCtrPoint))[k][0];
			(*(UCP[k]))[i][1] = (*(Uinter._pCtrPoint))[k][1];
			(*(UCP[k]))[i][2] = (*(Uinter._pCtrPoint))[k][2];
		}
	}

	std::cout << "Terminated u directional interpolation" << std::endl;	
	
	surfaceu << icp << ' ' << num_curve << std::endl;
	
	for(i = 0  ; i < num_curve; i++)
		surfaceu << *(UCP[i]) << "endpoint" << std::endl;

	surfaceu << "u-knot" << '\n' << *(LSC[0]._pKnot) << "endknot" << std::endl;
	surfaceu << "v-knot" << '\n' << *(Uinter._pKnot) << "endknot" << std::endl;
}


void SaveSRF(BsSurf& SRF)
{
	surface << SRF._n << " " << SRF._p << " " 
		    << SRF._m << " " << SRF._q << " " << std::endl;

	surface << SRF.m_pUdir->_pKnot->GetSize() << std::endl;
	surface << *(SRF.m_pUdir->_pKnot) << std::endl;

	surface << SRF.m_pVdir->_pKnot->GetSize() << std::endl;
	surface << *(SRF.m_pVdir->_pKnot) << std::endl;

	surface << SRF.m_pXcoord->GetRowSize() << " " 
		    << SRF.m_pXcoord->GetColSize() << std::endl;
	surface << *(SRF.m_pXcoord) << std::endl;

	surface << SRF.m_pYcoord->GetRowSize() << " " 
		    << SRF.m_pYcoord->GetColSize() << std::endl;
	surface << *(SRF.m_pYcoord) << std::endl;

	surface << SRF.m_pZcoord->GetRowSize() << " " 
		    << SRF.m_pZcoord->GetColSize() << std::endl;
	surface << *(SRF.m_pZcoord) << std::endl;
}

