#ifndef _MATRIX_H_
#define _MATRIX_H_


#include <assert.h>
#include <iostream> 
#include <iomanip>
#include <fstream> 

#ifdef HAVE_GSL_GSL_MATRIX_H 

	#include <gsl/gsl_matrix.h>
	
	#define DECLARE_GSL_FUCTION() void ConvertGslMatrix(gsl_matrix* Mout) const;\
			                      void InsertCol(const gsl_vector* v, int col);\
			                      friend void SolverLU(const Matrix& LHM, Matrix& RHM); //
//#ifdef 와 #else 사이는 공백줄 한개로 구분..반드시 한칸 띄운다.
#else 

	#define DECLARE_GSL_FUCTION() 

#endif
	
#include "vector.h"

/**********************************************************************************************/
/*           Copyleft by Jo, Joon woo                                                         */
/*           2001. 12.                                                                        */
/*                                                                                            */
/*           Provided functions list                                                          */
/*           element accessing operation by [][]                                              */
/*           element accessing operation by ( , ) with index error bounding                   */
/*           Matrix operation + and +=                                                        */
/*           Matrix operation- and -=                                                         */
/*           Matrix operation* and *= (*= is only able to use equal size square matrix both)  */
/*           Matrix and Vector product *                                                      */
/*           Matrix Transpose                                                                 */
/*           Insertion and Extraction of row and those of col                                 */
/*           Row swapping                                                                     */
/*           Output setting                                                                   */
/**********************************************************************************************/


class Matrix 
{
private:
	int m_RowSize, m_ColSize;
	double** p;
	friend class Vector;
	static bool m_SciFiFlag;
	
public:
	Matrix(int r, int c);    //constructor
	Matrix(const Matrix& m); //copy constructor
	Matrix(const Vector& v); //constructor with Vector object
	virtual ~Matrix(void);			 //destructor
	double& operator()(int i, int j);
	double* operator[](int i) { return (p[i]); }
	
	//Matrix function
	Matrix Transpose(void);
	void ClearAll(void);
	void ClearAll2(void);
	//double Det(void); 
	int GetRowSize(void) const { return (m_RowSize); }
	int GetColSize(void) const { return (m_ColSize); }
	
	void InsertRow(const Vector& v, int row);
	void InsertRow2(const Vector& v, int row);
	void InsertRow3(const Vector& v, int row); 
	void InsertCol(const Vector& v, int col);

	void AddCol(const Vector& v);
	void AddMat(const Matrix& m);
	void JoinMat(const Matrix& m);
	void CutRow(int num);
	void LoadIdentity(void);
	Vector ExtractRow(int row);
	Vector ExtractCol(int col);
	void SwapRow(int r1, int r2);
	//static void SetSciFi(void) { m_SciFiFlag = true; }
	//static void UnSetSciFi(void) { m_SciFiFlag = false; }
	static void SetSciFi(void);
	static void UnSetSciFi(void);

	//algebraic operator
	Matrix& operator=(const Matrix& m);
	Matrix& operator+=(const Matrix& m);
	Matrix  operator+(const Matrix& m);
	Matrix& operator-=(const Matrix& m);
	Matrix  operator-(const Matrix& m);
	Matrix  operator*(const Matrix& m);
	Vector  operator*(const Vector& v);
	Matrix& operator*=(const Matrix m);

	Matrix  operator^(int i);
	friend Matrix operator*(const Matrix& m , double d);
	friend Matrix operator*(double d , const Matrix& m);

	//input, output stream operator
	friend std::ostream&  operator<< ( std::ostream&   out, Matrix& m );
	friend std::ofstream& operator<< ( std::ofstream& fout, Matrix& m);


	DECLARE_GSL_FUCTION();

};

#endif

