#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <math.h>
#include "vector.h"

bool Vector::m_SciFiFlag;
//void Vector::SetSciFi(void);
//void Vector::UnSetSciFi(void);

void Vector::SetSciFi(void)
{
	m_SciFiFlag = true;
}

void Vector::UnSetSciFi(void)
{
	m_SciFiFlag = false;
}

//벡터의 요소 개수와 column 인지 row vector인지 를 받아서 벡터 객체를 생성
Vector::Vector(int e, bool iscol) : m_Size(e), m_bIsCol(iscol)
{
	//요소가 1개면 scalar기 때문에 assert
	//빈 벡터에 값을 체울수있도록 하기 위해 스칼라도 요소1개의
	//벡터로 인정 2002.7.29
	//assert(e > 1);
	assert(e > -1);
	
	if (e==0) p = '\0';
	else
	{
		p = new	double [e];
	
		for(int i = 0; i < e; ++i)
			p[i] = 0.0;
	}
}

//복사 생성자
Vector::Vector(const Vector& v) : m_Size(v.m_Size), m_bIsCol(v.m_bIsCol)
{
	p = new double [v.m_Size];
	
	for(int i = 0 ; i < m_Size ; i++)
		p[i] = v.p[i];
}

Vector::Vector(const double* d, int size, bool iscol) : m_Size(size), m_bIsCol(iscol)
{
	p = new double [m_Size];
	
	for(int i = 0 ; i < m_Size ; i++)
		p[i] = d[i];
}

Vector::~Vector(void)
{
	delete [] p;
}

double& Vector::operator[](int i) 
{ 
	//indexing error
	if(i < 0 || i >= m_Size)
	{
		std::cerr << "illegal vect index " << i << std::endl;
		assert(false);
	}

	return (p[i]); 
}

//대입 연산자 오버로딩
Vector& Vector::operator=(const Vector& v)
{
	//assert안에 expression이 참이면 통과
	assert(v.m_Size == m_Size);
	
	for(int i = 0; i < m_Size; ++i)
		p[i] = v.p[i];

	m_bIsCol = v.m_bIsCol;

	return (*this);
}

//+ 연산자 오버로딩
Vector Vector::operator+(const Vector& v)
{
	assert(v.m_Size == m_Size);
	
	Vector ans(m_Size);

	for(int i = 0 ; i < m_Size; i++)
		ans[i] = p[i] + v.p[i];

	return (ans);
}

//+= 연산자 오버로딩
Vector& Vector::operator+=(const Vector& v)
{
	assert(v.m_Size == m_Size);

	for(int i = 0 ; i < m_Size ; i++)
		p[i] += v.p[i];

	return (*this);
}

//- 연산자 오버로딩
Vector Vector::operator-(const Vector& v)
{
	assert(v.m_Size == m_Size);
	
	Vector ans(m_Size);

	for(int i = 0 ; i < m_Size; i++)
		ans[i] = p[i] - v.p[i];

	return (ans);
}

//-= 연산자 오버로딩
Vector& Vector::operator-=(const Vector& v)
{
	assert(v.m_Size == m_Size);

	for(int i = 0 ; i < m_Size ; i++)
		p[i] -= v.p[i];
	
	return (*this);
}

//내적을 위한  * 연산자 오버로딩
double Vector::operator*(const Vector& v)
{
	assert(v.m_Size == m_Size);
	
	double dot = 0.0;

	for(int i = 0 ; i < m_Size ; i++)
		dot += p[i]*(v.p[i]);

	return (dot);
}

//vector * matrix 에 쓰이는 * 연산자 오버로딩 결과는 행렬
Matrix  Vector::operator*(const Matrix& m)
{
	//벡터가 앞에서 곱해지기 때문에 벡터는 row 벡터여야 하고 사이즈와 m.RowSize가 
	//같아야한다.
	assert(!m_bIsCol && m_Size == m.m_RowSize);
	
	//현재 벡터와 같은 Dimension의 행렬을 생성하고
	Matrix temp((*this));

	//결과 행렬을 만들고 
	Matrix result(1, m.m_ColSize);
	
	//오버로딩해둔 행렬 * 행렬 기능을 이용한다. temp는 현재 자기자신이 행렬로 변한것
	result = temp * m;

	return (result);
}

Vector operator*(const Vector& v , double d)
{
	Vector temp(v);

	for(int i = 0 ; i < v.m_Size ; i++)
		temp.p[i] = v.p[i] * d;

	return (temp);
}

Vector operator*(double d , const Vector& v)
{
	Vector temp(v);
	temp = v * d;
	return (temp);
}

Vector operator/(const Vector& v, double d)
{
	Vector temp(v);
	
	for(int i = 0; i < v.m_Size; i++)
		temp.p[i] = v.p[i] / d;

	return (temp);
}

Vector operator/(double d , const Vector& v)
{
	Vector temp(v);
	temp = v / d;
	return (temp);
}

Vector Vector::Transpose(void)
{
	Vector temp((*this));
	temp.m_bIsCol ^= true;

	return (temp);
}

double Vector::SquareSum(void)
{
	int i;
	double sum = 0.0;

	for(i = 0; i < m_Size; i++)
		sum += (p[i]) * (p[i]);

	return sum;
}

double Vector::Norm(void)
{
	int i;
	double sum = 0.0;

	for(i = 0; i < m_Size; i++)
		sum += (p[i]) * (p[i]);

	return sqrt(sum);
}

double Vector::MaxNorm(void)
{
	int i, n;
	double temp;
	n = m_Size;
	temp = fabs(p[0]);

	for(i = 1 ; i < n ; i++)
		temp = (temp < fabs(p[i])) ? fabs(p[i]) : temp;

	return temp;
}

std::ostream& operator << (std::ostream& out, Vector& v)
{
	assert(v.p != '\0');

	for(int i = 0 ; i < v.m_Size; i++)
	{
		if(v.m_SciFiFlag)
			out <<  std::setiosflags(std::ios::showpos) <<std::setiosflags(std::ios::scientific) << std::setprecision(7) << v.p[i] << "   " ;
		else
			out <<  std::setw(12) << std::setprecision(7) << v.p[i] << "   " ;
		
		if(v.m_bIsCol) out << std::endl;
	}
	out << std::resetiosflags(std::ios::showpos) << std::resetiosflags(std::ios::scientific);
	return out;	
}

std::ofstream& operator << (std::ofstream& fout, Vector& v)
{
	for(int i = 0 ; i < v.m_Size; i++)
	{
		if(v.m_SciFiFlag)
			fout << std::setiosflags(std::ios::showpos) << std::setiosflags(std::ios::scientific) << std::setprecision(7) << v.p[i] << "   " ;
		else
			fout << std::setw(12) << std::setprecision(7) << v.p[i] << "   " ;
		
		if(v.m_bIsCol) fout << std::endl;
	}
	fout << std::resetiosflags(std::ios::showpos) <<std::resetiosflags(std::ios::scientific);
	return fout;	
}

double DotProduct(Vector& a, Vector& b)
{
	assert(a.m_Size == b.m_Size);

	int i;
	double sum = 0.0;
	
	for(i = 0; i < a.m_Size ; i++)
		sum += a[i]*b[i];

	return sum;
}	

double Distance3D(Vector v1, Vector v2) 
{
	Vector v(3);
	v = v1 - v2;
	double a, b ,c ;
	a = v[0]*v[0];
	b = v[1]*v[1];
	c = v[2]*v[2];

	return sqrt(a+b+c);
	//return  sqrt((v[0]*v[0]) + (v[1]*v[1]) + (v[3]*v[3]));
}

void Vector::ClearAll(void)
{
	for (int i = 0; i < m_Size; i++)
		p[i] = 0.0;
}

void Vector::DelAll(void)
{
	if(p != '\0')
		delete [] p;
	m_Size = 0;
	p = '\0';
}

/* vector의 마지막 num개의 요소를 잘라내는 함수*/
void Vector::CutComponent(int num)
{
	assert(num <= m_Size);
	
	if(num == m_Size)
	{
		delete [] p;
		p = '\0';
		return;
	}

	int i;
	
	m_Size -= num;

	double* temp = new double[m_Size];

	for(i = 0; i < m_Size; i++)
		temp[i] = p[i];

	delete [] p;

	p = new double[m_Size];
	
	for(i = 0; i < m_Size; i++)
		p[i] = temp[i];

	delete [] temp;
}

//dellist에 들어있는 인덱스에 해당하는 값을 다 지운다.
//단 인덱스값은 정렬되어서 넘겨진다고 가정
void Vector::DelComponent(Vector& dellist)
{
	int i,j,k;

	double* temp = new double[m_Size-dellist.GetSize()];

	k = j = 0;
	
	for(i = 0 ; i < dellist.GetSize(); i++)
	{
		//지워야하는 인덱스 바로 앞까지 복사
		while(j < dellist[i])
		{
			temp[k] = p[j];
			j++; k++;
		}
		//빠져나오면 지워야 하는 인덱스에 와있으므로 하나 증가
		j++;
	}

	for( i = j ; i < m_Size; i++, k++)
		temp[k] = p[i];

	
	delete [] p;

	m_Size -= dellist.GetSize();

	p = new double[m_Size];

	for(i = 0; i < m_Size; i++)
		p[i] = temp[i];
}

void Vector::AddComponent(double com)
{
	
	int i;
	
	//special case
	if(m_Size == 0)
	{
		m_Size++;
		delete [] p;
		p = new double[m_Size];
		p[0] = com;
		return;
	}
	
	double* temp = new double[m_Size];

	for(i = 0; i < m_Size; i++)
		temp[i] = p[i];
	

	delete [] p;

	m_Size++;

	p = new double[m_Size];
	
	for(i = 0; i < m_Size-1; i++)
		p[i] = temp[i];

	p[m_Size-1] = com;

	delete [] temp;
}



void Vector::Sort(int left, int right)
{
	assert(m_Size > 0);

	int i,j;
	double x,y;

	i = left;
	j = right;
	x = p[(left+right) / 2];
	
	do {
		while(p[i] < x && i < right) 
			i++;
		while(x < p[j] && j > left) 
			j--;

		if(i <= j) {
			y = p[i];
			p[i] = p[j];
			p[j] = y;
			i++;
			j--;
		}
	}while(i <= j);

	if(left<j) 
		Sort(left,j);
	if(i<right) 
		Sort(i,right);
}

//현재 벡터를 주어진 [left,right] 구간 사이로 선형 사상한다.
#ifdef HAVE_GSL_GSL_MATRIX_H
void Vector::Mapping(double left, double right)
{
	//[left 1 ; right 1]
	Matrix CO(2,2);
	CO[0][0] = p[0];         CO[0][1] = 1;
	CO[1][0] = p[m_Size-1];  CO[1][1] = 1;

	Vector R(2), S(2);
	R[0] = 0; R[1] = 1;

	S =  ((CO^(-1)) *  R);
	
	for(int i = 0; i < m_Size; i++)
	{
		//y = S[0]x+S[1]
		p[i] = S[0]*p[i] + S[1];
	}
}
#endif

