#ifndef _VECTOR_H_
#define _VECTOR_H_

#include <assert.h>
#include <iostream> 
#include <iomanip>
#include <fstream> 
#include "matrix.h"

/***************************************************************************************/
/*        Copyleft by Jo, Joonwoo                                                      */
/*        행렬과 함께 쓰일 Vector 클래스로써 vector 고유의 기능도 가지고 있고          */
/*        행렬의 행을 Pivoting 할때 각 행을 vector로 만들어 전환하기위해 만들어 졌다.  */
/*        가지고 있는 기능들                                                           */ 
/*        +, +=, -, -=,  내적                                                          */

//Column, Row vector를 결정해줄 열거자들 true, false 대신 의미를 명확히 하기 위해 쓰임
enum { ROW, COL }; 

class Vector
{
private :
	double* p;
	int m_Size;
	bool m_bIsCol;
	friend class Matrix;
	static bool m_SciFiFlag;

public:
	Vector(int e, bool iscol = COL); //constructor
	Vector(const double * d, int size, bool iscol = COL);
	Vector(const Vector& v); //copy constructor
	virtual ~Vector(void); //destructor
	double& operator[](int i); 

	//Vector function
	Vector Transpose(void);
	double SquareSum(void);
	double Norm(void);
	double MaxNorm(void);
	int GetSize(void) const { return (m_Size); }
	void ClearAll(void);
	void DelAll(void);
	void CutComponent(int num);
	void DelComponent(Vector& dellist);
	void AddComponent(double com);
	void Sort(int left, int right);
#ifdef HAVE_GSL_GSL_MATRIX_H	
	void Mapping(double left, double right);
#endif
	//static void SetSciFi(void) { m_SciFiFlag = true; }
	//static void UnSetSciFi(void) { m_SciFiFlag = false; }
	static void SetSciFi(void);
	static void UnSetSciFi(void);
	
	//algebraic function
	Vector& operator=(const Vector& v);
	Vector  operator+(const Vector& v);
	Vector& operator+=(const Vector& v);
	Vector  operator-(const Vector& v);
	Vector& operator-=(const Vector& v);
	double  operator*(const Vector& v);
	Matrix  operator*(const Matrix& m);
	friend Vector operator*(const Vector& v , double d);
	friend Vector operator*(double d , const Vector& v);
	friend Vector operator/(const Vector& v , double d);
	friend Vector operator/(double d , const Vector& v);
	
	//input, output stream operator		
	friend std::ostream& operator<< ( std::ostream& out, Vector& v );
	friend std::ofstream& operator<< ( std::ofstream& fout, Vector& v );

	//other friend 
	friend double DotProduct(Vector& a, Vector& b);
	friend double Distance3D(Vector v1, Vector v2);
};


#endif

